package com.xintong.visualinspection.dao.master;

import java.util.List;

import com.xintong.visualinspection.bean.UserClassRemove;
import org.apache.ibatis.annotations.Mapper;

import com.xintong.visualinspection.bean.UnregularUserClass;
import com.xintong.visualinspection.bean.UserClass;

/**
 * 文件名：UserInfoDao
 * 版本信息：日期：2017/3/30 Copyright 江苏省交通规划设计院 Corporation 2017 版权所有.
 */
@Mapper
public interface UserClassDao  {
    public List<UserClass> selectList(UserClass userClass);
    public void insert(UserClass userClass);
    public void insertremove(UserClassRemove userClassRemove);


    public void update(UserClass userClass);
    public void update_detail(UnregularUserClass unregularUserClass);


    public void delete(UserClass userClass);

    public void deleteById(List<String> ids);
    public String getUsersByTeamId(Integer user_class_id);
    
    public void insert_detail(List<UnregularUserClass> userClass);
    public void insert_detail2(UnregularUserClass userClass);

    public void deleteDetail(UserClass userClass);
    public void deleteDetailById(List<String> ids);

    public List<UserClass> getUsersClassByDeptAndWorkDay(UserClass userClass);

    public List<UnregularUserClass> getUnregularUsersClassByDeptAndWorkDay(UserClass userClass);
}
